Imports QuartzTypeLib

Friend Class ClsAudio
    Private MediaControl As FilgraphManager
    Private mIsPaused As Boolean = False
    Private mVolume As Integer = 0

    Private mReaded As New List(Of ClsFichier)
    Private mRead As New List(Of ClsFichier)
    Private mReading As ClsFichier = Nothing

    Private mThread As Threading.Thread
    Private mCanRun As Boolean = True

    Public Event NewFile(ByVal pNewFile As ClsFichier)

#Region "Construction et destruction"
    Public Sub New()
        Randomize(Environment.TickCount)
        mThread = New Threading.Thread(AddressOf SubThread)
        mThread.Name = "Repetition de musique"
        mThread.Start()
    End Sub
    Protected Overrides Sub Finalize()
        MusicStop()
        MyBase.Finalize()
    End Sub
#End Region

#Region "proprits publiques"
    Public ReadOnly Property ToRead() As List(Of ClsFichier)
        Get
            Return mRead
        End Get
    End Property
    Public ReadOnly Property Readed() As List(Of ClsFichier)
        Get
            Return mReaded
        End Get
    End Property
    Public ReadOnly Property Reading() As ClsFichier
        Get
            Return mReading
        End Get
    End Property
    Public ReadOnly Property IsPaused() As Boolean
        Get
            Return mIsPaused
        End Get
    End Property
#End Region

#Region "proprit prives"

    Private ReadOnly Property BasicAudio() As IBasicAudio
        Get
            Return CType(MediaControl, IBasicAudio)
        End Get
    End Property

    Private ReadOnly Property MediaEvent() As IMediaEvent
        Get
            Return CType(MediaControl, IMediaEvent)
        End Get
    End Property

    Private ReadOnly Property MediaPosition() As IMediaPosition
        Get
            Return CType(MediaControl, IMediaPosition)
        End Get
    End Property
#End Region

#Region "membres publiques"

    Public Function Paused() As Boolean
        Paused = IsPaused
    End Function

    Public Sub Start(Optional ByVal pFile As ClsFichier = Nothing)
        Try
            If MediaControl IsNot Nothing Then
                MusicStop()
            End If
            SyncLock mRead
                If pFile Is Nothing Then
                    Do While True
                        If mRead.Count > 2 Then
                            Dim lRnd As Integer
                            Do
                                lRnd = CInt(Rnd() * mRead.Count)
                            Loop While lRnd >= mRead.Count AndAlso mRead.Count > 0
                            mReading = mRead(lRnd)
                            If mReading.Readed Then
                                mRead.Remove(mReading)
                                SyncLock mReaded
                                    mReaded.Add(mReading)
                                End SyncLock
                                mReading = Nothing
                            Else
                                If mReading.Exist = False Then
                                    mReading = Nothing
                                    Continue Do
                                Else
                                    Exit Do
                                End If
                            End If
                        ElseIf mRead.Count = 0 Then
                            If mReaded.Count > 0 Then
                                SyncLock mReaded
                                    mRead.AddRange(mReaded)
                                    mReaded.Clear()
                                End SyncLock
                                For Each lFic As ClsFichier In mRead
                                    lFic.Readed = False
                                Next
                                If mRead.Count > 0 Then
                                    Continue Do
                                Else
                                    Exit Sub
                                End If
                            Else
                                Exit Sub
                            End If
                        Else
                            mReading = mRead(mRead.Count - 1)
                            If mReading.Readed Then
                                mRead.Remove(mReading)
                                SyncLock mReaded
                                    mReaded.Add(mReading)
                                End SyncLock
                                mReading = Nothing
                            Else
                                If mReading.Exist = False Then
                                    mReading = Nothing
                                    Continue Do
                                Else
                                    Exit Do
                                End If
                            End If
                        End If
                    Loop
                Else
                    mReading = pFile
                End If
                If mReading.Exist = False Then
                    Exit Sub
                End If
                Do While mRead.IndexOf(mReading) <> -1
                    mRead.Remove(mReading)
                Loop
                Do While mReaded.IndexOf(mReading) <> -1
                    mReaded.Remove(mReading)
                Loop
                AddLog("ClsAudio", "Lecture du fichier """ + mReading.FileName + """", EPrio.Normal)
                mReaded.Add(mReading)
                mReading.Readed = True
                MediaControl = New FilgraphManager
                MediaControl.RenderFile(mReading.FileName)
                BasicAudio.Volume = mVolume
                mIsPaused = False
                MediaControl.Run()
                RaiseEvent NewFile(mReading)
                System.GC.Collect()
            End SyncLock
        Catch ex As Exception
            AddLog("ClsAudio.Start", "Exception non gre : " + ex.ToString, EPrio.Avertissement)
#If DEBUG Then
            Debug.Print(ex.ToString)
            Stop
#End If
        End Try
    End Sub

    Public Sub Pause()
        If (MediaControl IsNot Nothing) And Not IsPaused Then
            mIsPaused = True
            MediaControl.Pause()
        End If
    End Sub

    Public Sub MusicResume()
        If (MediaControl IsNot Nothing) And IsPaused Then
            mIsPaused = False
            MediaControl.Run()
        End If
    End Sub

    Public Property Volume() As Integer
        Get
            Return mVolume
        End Get
        Set(ByVal value As Integer)
            If MediaControl IsNot Nothing Then
                mVolume = value
                BasicAudio.Volume = mVolume
            End If
        End Set
    End Property

    Public Sub MusicStop()
        SyncLock (Me)
            If MediaControl IsNot Nothing Then
                If mReading IsNot Nothing AndAlso mReaded.IndexOf(mReading) = -1 Then
                    mReaded.Add(mReading)
                    mReading = Nothing
                End If
                MediaControl.Stop()
                MediaControl = Nothing
                mIsPaused = False
            End If
        End SyncLock
    End Sub

    Public ReadOnly Property TimeElapsed() As Double
        Get
            If MediaControl IsNot Nothing Then
                Return MediaPosition.CurrentPosition
            Else
                Return 0
            End If
        End Get
    End Property

    Public ReadOnly Property TimeTotal() As Double
        Get
            If MediaControl IsNot Nothing Then
                Return MediaPosition.Duration
            Else
                Return 0
            End If
        End Get
    End Property

    Public Sub StopThread()
        If mThread IsNot Nothing Then
            mCanRun = False
            Threading.Thread.Sleep(200)
        End If
        If mThread IsNot Nothing Then
            mThread.Abort()
        End If
    End Sub
#End Region

    Private Sub SubThread()
        Try
            Dim lNb As Integer = 0
            SyncLock mRead
                For i As Integer = 0 To mRead.Count - 1
                    If i > mRead.Count - 1 Then Exit For
                    If mRead(i).Readed Then
                        SyncLock mReaded
                            mReaded.Add(mRead(i))
                            mRead.RemoveAt(i)
                            i -= 1
                        End SyncLock
                    End If
                Next
            End SyncLock
            Do While mCanRun
                Dim lMediaControl As FilgraphManager = MediaControl
                If lMediaControl IsNot Nothing Then
                    If DirectCast(lMediaControl, IMediaPosition).CurrentPosition >= DirectCast(lMediaControl, IMediaPosition).Duration - 0.2 Then
                        If lNb > 2 Then
                            MusicStop()
                            Start()
                            lNb = 0
                            SyncLock mRead
                                For i As Integer = 0 To mRead.Count - 1
                                    If i > mRead.Count - 1 Then Exit For
                                    If mRead(i).Readed Then
                                        SyncLock mReaded
                                            mReaded.Add(mRead(i))
                                            mRead.RemoveAt(i)
                                            i -= 1
                                        End SyncLock
                                    End If
                                Next
                            End SyncLock
                        Else
                            lNb += 1
                        End If
                    Else
                        lNb = 0
                    End If
                ElseIf mRead.Count > 0 Then
                    If lNb > 2 Then
                        MusicStop()
                        Start()
                        lNb = 0
                        SyncLock mRead
                            For i As Integer = 0 To mRead.Count - 1
                                If i > mRead.Count - 1 Then Exit For
                                If mRead(i).Readed Then
                                    SyncLock mReaded
                                        mReaded.Add(mRead(i))
                                        mRead.RemoveAt(i)
                                        i -= 1
                                    End SyncLock
                                End If
                            Next
                        End SyncLock
                    Else
                        lNb += 1
                    End If
                End If
                Threading.Thread.Sleep(100)
            Loop
        Catch ex As Exception
            AddLog("ClsAudio.SubThread", "Exception non gre : " + ex.ToString, EPrio.Avertissement)
#If DEBUG Then
            Debug.Print(ex.ToString)
#Else
            MessageBox.Show(ex.ToString, "Erreur dans le thread", MessageBoxButtons.OK, MessageBoxIcon.Error)
#End If
        Finally
            mThread = Nothing
        End Try
    End Sub
End Class
